import { nativeRaf } from './dom';
import { removeArrayItem } from './util';
export class DomController {
    constructor() {
        this.r = [];
        this.w = [];
    }
    read(fn, ctx) {
        const task = !ctx ? fn : fn.bind(ctx);
        this.r.push(task);
        this.queue();
        return task;
    }
    write(fn, ctx) {
        const task = !ctx ? fn : fn.bind(ctx);
        this.w.push(task);
        this.queue();
        return task;
    }
    cancel(task) {
        return removeArrayItem(this.r, task) || removeArrayItem(this.w, task);
    }
    queue() {
        if (!this.q) {
            this.q = true;
            nativeRaf(timestamp => {
                this.flush(timestamp);
            });
        }
    }
    flush(timestamp) {
        let err;
        let task;
        try {
            while (task = this.r.shift()) {
                task(timestamp);
            }
            while (task = this.w.shift()) {
                task(timestamp);
            }
        }
        catch (e) {
            err = e;
        }
        this.q = false;
        if (this.r.length || this.w.length) {
            this.queue();
        }
        if (err) {
            throw err;
        }
    }
}
//# sourceMappingURL=dom-controller.js.map