(function (factory) {
    if (typeof module === 'object' && typeof module.exports === 'object') {
        var v = factory(require, exports); if (v !== undefined) module.exports = v;
    }
    else if (typeof define === 'function' && define.amd) {
        define(["require", "exports", './dom', './util'], factory);
    }
})(function (require, exports) {
    "use strict";
    var dom_1 = require('./dom');
    var util_1 = require('./util');
    var DomController = (function () {
        function DomController() {
            this.r = [];
            this.w = [];
        }
        DomController.prototype.read = function (fn, ctx) {
            var task = !ctx ? fn : fn.bind(ctx);
            this.r.push(task);
            this.queue();
            return task;
        };
        DomController.prototype.write = function (fn, ctx) {
            var task = !ctx ? fn : fn.bind(ctx);
            this.w.push(task);
            this.queue();
            return task;
        };
        DomController.prototype.cancel = function (task) {
            return util_1.removeArrayItem(this.r, task) || util_1.removeArrayItem(this.w, task);
        };
        DomController.prototype.queue = function () {
            var _this = this;
            if (!this.q) {
                this.q = true;
                dom_1.nativeRaf(function (timestamp) {
                    _this.flush(timestamp);
                });
            }
        };
        DomController.prototype.flush = function (timestamp) {
            var err;
            var task;
            try {
                while (task = this.r.shift()) {
                    task(timestamp);
                }
                while (task = this.w.shift()) {
                    task(timestamp);
                }
            }
            catch (e) {
                err = e;
            }
            this.q = false;
            if (this.r.length || this.w.length) {
                this.queue();
            }
            if (err) {
                throw err;
            }
        };
        return DomController;
    }());
    exports.DomController = DomController;
});
//# sourceMappingURL=dom-controller.js.map